/* Standard Metropolis Monte Carlo update (Metropolis, Rosenbluth**2, 
   Teller**2, J. Chem. Phys. 21 (1953) 1087).  A new trial spin is 
   chosen at random, and accepted if 
   exp(- beta * change in energy) > a random number in [0,1).

   There is a separate version for zero temperature, since in that case
   there is no need to calculate an exponential and a random number 
   (the change is accepted if it does not increase the energy). However 
   the update needs to be done with the sites chosen at random rather 
   than in order, so as not to violate the ergodicity requirement.
*/


#include "comext.h"
#include <math.h>

metupdate(nmet)
int nmet;
{
  int imet,i,j;
  int posi,posj,negi,negj;
  short posi_spin,posj_spin,negi_spin,negj_spin;
  short old_spin,new_spin;
  double old_energy,new_energy,energy_diff;
  double pranf();

  num_accept = 0;
 
  for(imet=0;imet<nmet;imet++){

     for(i=0;i<isiz;i++){
     for(j=0;j<jsiz;j++){

	   old_spin = spin[i][j];

	   /* Choose a new trial spin */
	   new_spin = (pranf() * (Q-1)) + 1;
	   new_spin = (old_spin + new_spin) % Q;
	   /* new_spin = 1 - old_spin; */     /* only for Q=2 */

	   posi = plusi(1);
	   negi = minusi(1);
	   posj = plusj(1);
	   negj = minusj(1);
	   posi_spin = spin[posi][j];
	   negi_spin = spin[negi][j];
	   posj_spin = spin[i][posj];
	   negj_spin = spin[i][negj];

	   old_energy = 0.0;
	   if (old_spin == posi_spin)
	     old_energy += J[i][j][0];
	   if (old_spin == negi_spin)
	     old_energy += J[negi][j][0];
	   if (old_spin == posj_spin)
	     old_energy += J[i][j][1];
	   if (old_spin == negj_spin)
	     old_energy += J[i][negj][1];
	   
	   new_energy = 0.0;
	   if (new_spin == posi_spin)
	     new_energy += J[i][j][0];
	   if (new_spin == negi_spin)
	     new_energy += J[negi][j][0];
	   if (new_spin == posj_spin)
	     new_energy += J[i][j][1];
	   if (new_spin == negj_spin)
	     new_energy += J[i][negj][1];
	   
	   energy_diff = beta * (old_energy - new_energy);
	   /* reversed due to - sign in energy */
         
	   if ( (energy_diff<=0.0) || (exp(-energy_diff) > pranf() ) ) {
	      num_accept++;
	      spin[i][j] = new_spin;
           }

     }}  /* end of loop over sites */

  }  /* end of loop over sweeps */

}



/* Zero temperature Metropolis update. Change is accepted 
   if it does not increase the energy.
*/

metupdate_zero(nmet)
int nmet;
{
  int imet,i,j,n;
  int posi,posj,negi,negj;
  short posi_spin,posj_spin,negi_spin,negj_spin;
  short old_spin,new_spin;
  double old_energy,new_energy,energy_diff;
  double pranf();

  num_accept = 0;
 
  for (imet=0;imet<nmet;imet++) {

     for (n=0;n<sites;n++) {

           /* Chose a random site */
	   i = pranf() * isiz;
	   j = pranf() * jsiz;
	   if (i == isiz) i = 0;              /* in case pranf() returns 1 */
	   if (j == jsiz) j = 0;              /* in case pranf() returns 1 */

	   /* Choose a new trial spin */
	   old_spin = spin[i][j];
	   /* new_spin = 1 - old_spin; */     /* only for Q=2 */
	   new_spin = (pranf() * (Q-1)) + 1;
	   new_spin = (old_spin + new_spin) % Q;

	   posi = plusi(1);
	   negi = minusi(1);
	   posj = plusj(1);
	   negj = minusj(1);
	   posi_spin = spin[posi][j];
	   negi_spin = spin[negi][j];
	   posj_spin = spin[i][posj];
	   negj_spin = spin[i][negj];

	   old_energy = 0.0;
	   if (old_spin == posi_spin)
	     old_energy += J[i][j][0];
	   if (old_spin == negi_spin)
	     old_energy += J[negi][j][0];
	   if (old_spin == posj_spin)
	     old_energy += J[i][j][1];
	   if (old_spin == negj_spin)
	     old_energy += J[i][negj][1];
	   
	   new_energy = 0.0;
	   if (new_spin == posi_spin)
	     new_energy += J[i][j][0];
	   if (new_spin == negi_spin)
	     new_energy += J[negi][j][0];
	   if (new_spin == posj_spin)
	     new_energy += J[i][j][1];
	   if (new_spin == negj_spin)
	     new_energy += J[i][negj][1];
	   
	   energy_diff = old_energy - new_energy;  
	   /* reversed due to - sign in energy */
         
	   if ( energy_diff<=0.0 ) {
	      num_accept++;
	      spin[i][j] = new_spin;
	   }

     }  /* end of loop over sites */

  }  /* end of loop over sweeps */

}
